!------------------------------------------------
! extract_iii
!
! Usage:
!  extract_iii FlagCarto FlagHeader Lati Long DataFile ResuFile Mot1 ... MotN
!
!  FlagCarto  = 'TRUE'/'false' = genere des sorties au format carto
!  FlagHeader = 'TRUE'/'false' = genere un header pour DataFile
!  Lati       = bornes en latitude (latmin,latmax)
!  Long       = bornes en longitude (lonmin,lonmax)
!  Indtm      = indice terre/mer (0=mer,1=terre, 2=mer+terre)
!  Indjn      = indice jour+nuit (0=nuit,1=jour, 2=nuit+jour)
!  DataFile   = fichier contenant la liste des donnees
!  ResuFile   = fichier resultat (longueur=9!!!)
!  Mot1...N   = entiers designant le mot a extraire
!
! Creation :
! 12/11/2003 - O.Maury
!------------------------------------------------
! Modifications :
! 24/11/2003 - O.Maury : 
!   - ajout d'un header a ResuFile
!   - ajout des mots par defaut
!   - ajout sorties carto (ResuFile_III-*)
! 25/11/2003 - O.Maury : 
!   - ajout de la date dans mots par defaut (228)
! 26/11/2003 - O.Maury
!   - correction date : lue dans le header (228)
! 03/12/2003 - R. Armante
!   - passage des coordonnees d'une zone geographique
!     par defaut, -90,90 en latitude et -180,180 en longitude
!   - choix de l'indice terre/mer (terre+mer par defaut)
!   - choix de l'indice jour/nuit (jour/nuit par defaut)
!   - correction bug sur la date : lorsque l'on ecrit la date sous la forme
!     ddmmyyyy, on depasse la precision de la machine sur le nombre de chiffre
!     apres la virgule sur les flottants. 
!     ==> correction en ramenant la date sous la forme ddmmyy
!------------------------------------------------
! Definition des  variables
!
      PARAMETER (Nbval_Header=8,npt_iasi=8461)
      CHARACTER DataFile*50,ResuFile*9

      INTEGER MotDefaut(Nbval_Header)
      INTEGER MotDesire(500)
      INTEGER IndiceMot(500)

      LOGICAL DateIdentique

      REAL*4 ValeurMot(500),Corrige(500)
      REAL Latmin,Latmax,Lonmin,Lonmax
      REAL Indice_tm,Indice_jn
      real cor_ang(324,13),ang_zen(13)
      data ang_zen/00.00,05.72,09.19,12.93,16.69,20.41,24.04,27.85,31.67,35.28,39.18,43.47,47.14/

      CHARACTER*5 FlagCarto
      CHARACTER*5 FlagHeader
      CHARACTER*20 Lati,Longi
      CHARACTER*4 arg
      CHARACTER*3 extract
      CHARACTER*8 Suff
      CHARACTER*15 carac

! 29 = latitude ()
! 30 = longitude ()
! 31 = local zenith angle for the box ()
! 35 = surface elevation (m)
! 36 = identification of cloud test
! 37 = time (hhmmss)
! 39 = land/sea + day/night flag
! 228 ou 230 (du header)= date du spot (ddmmyy)
! Rq: le mot 228 est celui du header, pas des donnees
      DATA MotDefaut /0,0,0,0,0,0,0,0/
!     DATA MotDefaut /29,30,31,35,36,37,39,228/
! Declaration partie Airs
INTEGER,PARAMETER :: NCHANNELMAXIASI=8461,NCHANNELMAXAMSU=15
INTEGER  yy,mm,dd,hh,mn,ss,yyyymmd,hhmnss

REAL IASI_Time,IASI_Latitude,IASI_Longitude, &
     IASI_Satellite_Zenith,IASI_Solar_Zenith,Cloud_Cover
INTEGER Land_Sea_Alti
REAL ,DIMENSION(NCHANNELMAXIASI) :: IASI_Radiances
REAL*8  date_iasi
integer Orbit_number
character*4 IASI_Quality_Flag

REAL ,DIMENSION(NCHANNELMAXAMSU) :: AMSU_BT
REAL  AMSU_Quality_Flag

      CHARACTER*100 Nom_buffer !nom du fichier buffer  lire
      CHARACTER*100 SORTIE !sortie ascii
      CHARACTER*5 process !nom de l'action  effectuer : NCEP,Prcom,STRAT,DAO
      CHARACTER*3 CGRANULE,CJOUR
      CHARACTER*2 CHEUREDEB,CHEUREFIN,CMINDEB,CMINFIN,CANNEE
      CHARACTER*40 PREFIXE,SUFFIXE,PATH_DATA
      INTEGER Ngranule,ANNEE,HEUREDEB,MINDEB,HEUREFIN,MINFIN,JOUR
      real freq_iasi(8461),tb_hs(8461)
!**Test ciel clair**
      integer test_ciel_clair, couv_nuage
!**Test terre/mer**
      integer*4 indtermer(2160,1080),flagtermer,terre_mer,inittm,iunittm
      character*200 nomfichtermer

      INTEGER ii

      include 'iasidatafile.h'
! definition des canaux HS
      do i=1,npt_iasi
         tb_hs(i)=0
      enddo
! canaux 128 et 263 morts a partir de mars 2003
!     tb_hs(116)=1.
      do i=1,npt_iasi
         if(tb_hs(i).eq.1) write(*,*)'canal',i, &
         ' HS ==> flag quality pas pris en compte'
      enddo
!***Lecture des frequences des canaux***
      do i=1, NCHANNELMAXIASI
         freq_iasi(i)=645.+(i-1)*0.25
      enddo

!***Terre/mer***
      nomfichtermer=trim(racine_iasidata)//'/land_sea_alti_map.dat'
      inittm = 0
      iunittm = 1
!*CHARGEMENT DE LA BANQUE TERRE/MER DANS LE TABLEAU indtermer*
      CALL sub_land_sea_alti(0.,0.,inittm,terre_mer,iunittm   &
                          ,indtermer,nomfichtermer)
!*DRAPEAU D'UTILISATION DE LA BANQUE TERRE/MER*
      inittm = 1

!***Test ciel clair***
!     test_ciel_clair=couv_nuage   &
!              (0,AIRS_Radiances,AMSU_BT   &
!              ,1,1,freq_airs,0)
      cloud_cover=-1.

!------------------------------------------------
! Lecture arguments (pas de test)
!
      CALL getarg(1,FlagCarto)
      CALL getarg(2,FlagHeader)
      CALL getarg(3,Lati)
      CALL getarg(4,Longi)
      CALL getarg(5,arg)
      read(arg,*) Indice_tm
      CALL getarg(6,arg)
      read(arg,*) Indice_jn
      CALL getarg(7,DataFile)
      CALL getarg(8,ResuFile)

      IFIN = iargc()
      NbrDesire = IFIN-8
      DO ind = 9,IFIN
         CALL getarg(ind,arg)
         read (arg,*) Mot
         MotDesire(ind-8)=Mot
      END DO
  
!      print *,FlagCarto
!      print *,FlagHeader
!      print *,DataFile
!      print *,ResuFile
!      print *,NbrDesire
       do indi=1,len(Lati)
          if(Lati(indi:indi).eq.',') Ind_lati=indi
       enddo
       do indi=1,len(Longi)
          if(Longi(indi:indi).eq.',') Ind_long=indi
       enddo
       read(Lati(1:Ind_lati-1),*) Latmin
       read(Lati(Ind_lati+1:len(Lati)),*) Latmax
       read(Longi(1:Ind_long-1),*) Lonmin
       read(Longi(Ind_long+1:len(Longi)),*) Lonmax
    
!------------------------------------------------
! Ajout mots par defaut
!
      DO ii=1,Nbval_Header
         IndiceMot(ii)=MotDefaut(ii)
      END DO
      DO jj=1,NbrDesire
         ii=jj+Nbval_Header
         IndiceMot(ii)=MotDesire(jj)
      END DO
      NbrMots = NbrDesire+Nbval_Header

!      print *,NbrMots
!      print *,IndiceMot

!------------------------------------------------
! Ouverture des fichiers
!
      close(11)
      open(unit=10, file = DataFile)
      open(unit=20, file = ResuFile, form='unformatted')
      open(599,file='fort.599',form='unformatted')
! Option Carto ?
      IF (FlagCarto .eq. 'TRUE') THEN
         DO iCarto=1,NbrDesire
            write(Suff,FMT='(A5,I3.3)') '_III-',MotDesire(iCarto)
            open(unit=(40+iCarto), file = ResuFile // Suff, &
                 form='unformatted')
         END DO
      END IF         

!------------------------------------------------
! Ecriture du header
!
      IF (FlagHeader .eq. 'TRUE') THEN
!         print *, NbrMots,(IndiceMot(ind),ind=1,NbrMots)
         WRITE (20) NbrMots,(IndiceMot(ind),ind=1,NbrMots)
         WRITE (*,*) 'Header',NbrMots,(IndiceMot(ind),ind=1,NbrMots)
      END IF

!================================================
! Debut boucle sur les fichiers de donnees
!================================================
 5    CONTINUE
!------------------------------------------------
! Lit le nom du fichier courrant puis l'ouvre 
!
      read(10,'(a100)', END = 15) Nom_buffer
!lecture iasi bufr
      open(1111,file=Nom_buffer,form='formatted')
          read(1111,*)
          read(1111,*)
          read(1111,*)
          read(1111,*) date_iasi
          yyyymmdd=int(date_iasi)
          hhmnss=int((date_iasi-yyyymmdd*1.)*1000000.)
          write(*,*) date_iasi,hhmnss
          read(1111,*) Orbit_number
          read(1111,*) IASI_Quality_Flag
          if(IASI_Quality_Flag.ne.'good') then
             close(1111)
             goto 5
          endif
          read(1111,*) Rlat
          read(1111,*) Rlon
          read(1111,*) IASI_Satellite_Zenith
          read(1111,*) IASI_Satellite_Azimuth
          read(1111,*) IASI_Solar_Zenith
          read(1111,*) IASI_Solar_Azimuth
          read(1111,*) IASI_Satellite_Alt
          read(1111,*) IASI_Cloud_Free
! skip avhrr informations
          do i=1,18
             read(1111,*)
          enddo
          read(1111,*) wavenb_deb
          read(1111,*) wavenb_fin
          read(1111,*) 
          read(1111,*) 
          do i=1,npt_iasi
             read(1111,*) IASI_Radiances(i)
          enddo
       close(1111)

       nbatm_tot=0
       nbatm_clear=0
!      DO I=1,number_of_golf_balls
          extract='oui'
          CALL sub_land_sea_alti(IASI_Latitude,IASI_Longitude   &
                 ,inittm,Land_Sea_Alti,iunittm,indtermer,nomfichtermer)
            indtm=0
            if(Land_Sea_Alti.ge.0) indtm=1
!          cloud_cover=couv_nuage(1,IASI_Radiances,AMSU_BT   &
!                   ,I,J,freq_airs,indtm)
!          if(cloud_cover.ne.0) extract='non'
          Do j=1,NbrMots+1
             ValeurMot(j)=0.
          Enddo
! determination de l'angle le plus proche
          angmin=99.99
          do iiang=1,13
             dist=abs(IASI_Satellite_Zenith-ang_zen(iiang))
             if(dist.lt.angmin) then
                 angmin=dist
                 iiangmin=iiang
             endif
          enddo
!         Do J=1,IASI_FOVS
         if(Rlat.lt.Latmin.or.Rlat.gt.Latmax) extract='non'
         if(Rlon.lt.Lonmin.or.Rlon.gt.Lonmax) extract='non'
          if(extract.eq.'oui') then
          indtm=0
          if(Land_Sea_Alti.ge.0) indtm=1
          endif
          indjn=1
          if(IASI_Solar_Zenith.ge.90) indjn=0
         if(indtm.ne.Indice_tm.and.Indice_tm.ne.2) extract='non'
         if(indjn.ne.Indice_jn.and.Indice_jn.ne.2) extract='non'
         if(extract.eq.'oui') then
          ValeurMot(1)=Rlat
          ValeurMot(2)=Rlon
          ValeurMot(3)=IASI_Satellite_Zenith
          ValeurMot(4)=Land_Sea_Alti
          ValeurMot(5)=cloud_cover
!         call seconds_to_calender(dble(IASI_Time/1000.),yy,mm,dd,hh,mn,ss)
          ValeurMot(6)=hhmnss
          ValeurMot(7)=float(indtm*10+indjn)
          ValeurMot(8)=yyyymmdd
          DO ind = 1, NbrDesire
        if(tb_hs(MotDesire(ind)).ne.1.and.IASI_Quality_Flag.ne.'good') &
           write(*,*) MotDesire(ind),IASI_Quality_Flag
            if(MotDesire(ind).le.npt_iasi.and.IASI_Quality_Flag.ne.'good') extract='non'
            if(MotDesire(ind).le.npt_iasi.and.IASI_Quality_Flag.eq.'good') then
!              ValeurMot(Nbval_Header+ind)= IASI_Radiances(MotDesire(ind))
               ValeurMot(Nbval_Header+ind)=tbrillans(freq_iasi(MotDesire(ind)), &
                        1e02*IASI_Radiances(MotDesire(ind)))
            endif
!           if(MotDesire(ind).gt.npt_iasi.and.AMSU_Quality_Flag(MotDesire(ind)-npt_iasi,1,I).ne.0) extract='non'
!           if(MotDesire(ind).gt.npt_iasi.and.AMSU_Quality_Flag(MotDesire(ind)-npt_iasi,1,I).eq.0) &
!              ValeurMot(Nbval_Header+ind)=AMSU_BT(MotDesire(ind)-npt_iasi,1,I) 
!           endif
          END DO
        ValeurMot(NbrMots+1)=IASI_Solar_Zenith

          nbatm_tot=nbatm_tot+1
         if(extract.eq.'oui') then
            nbatm_clear=nbatm_clear+1
            WRITE (20) (ValeurMot(ind),ind=1,NbrMots+1),Orbit_number
         endif

!        WRITE (20) (ValeurMot(ind),ind=1,NbrMots+1)
! Option Carto ?
         IF (FlagCarto .eq. 'TRUE') THEN
            DO iCarto=1,NbrDesire
               write(40+iCarto) ValeurMot(1),ValeurMot(2), &
                    ValeurMot(iCarto+Nbval_Header)
            END DO
         END IF
         end if
       write(*,*) 'Nb clear',nbatm_clear,nbatm_tot

!================================================
! Fin boucle
!================================================
      GO TO 5
 15   CONTINUE

!------------------------------------------------
! Fermeture des fichiers
!
      close (10)
      close (20)

      IF (FlagCarto .eq. 'TRUE') THEN
         DO iCarto=1,NbrDesire
            close(40+iCarto)
         END DO
      END IF
!------------------------------------------------
! FIN PROGRAM extract_iii
!
! 777  CONTINUE

      END
